"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ora = require('ora');
const chalk_1 = require("chalk");
const exec_1 = require("./exec");
class TaskContext extends exec_1.ShellContext {
    constructor(task, globalOptions) {
        super();
        this.task = task;
        this.globalOptions = globalOptions;
    }
    get options() {
        return this.task.options || {};
    }
}
exports.TaskContext = TaskContext;
class TaskManager {
    constructor() {
        this._tasks = {};
        this.globalOptions = {
            logLevel: 'debug',
            loading: true,
        };
    }
    getTasks() {
        return Object.keys(this._tasks)
            .map(k => this._tasks[k]);
    }
    addTask(task) {
        if (this._tasks[task.name]) {
            throw new TypeError(`Task name [${task.name}] already exists, please choose another task name!`);
        }
        this._tasks[task.name] = task;
        return task;
    }
    run(name = 'default', { options = null, args = [] }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this._tasks.all = this._tasks.all || task('all', Object.keys(this._tasks));
            this._tasks.default = this._tasks.default || this._tasks.all;
            const t = this._tasks[name];
            if (!t) {
                throw new TypeError(`Cannot find task with name [${name}]`);
            }
            if (t.dependencies) {
                let asyncDeps = [];
                let syncDeps = [];
                t.dependencies.forEach(t => {
                    if (t.async) {
                        asyncDeps.push(t);
                    }
                    else {
                        syncDeps.push(t);
                    }
                });
                yield Promise.all([
                    (() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        for (const t of syncDeps) {
                            yield this.run(t.name, { options: t.options });
                        }
                    }))(),
                    Promise.all(asyncDeps.map(t => this.run(t.name, { options: t.options }))),
                ]);
            }
            let ld;
            let text = `${t.name}`;
            t.options = options;
            t.args = args;
            let ctx = new TaskContext(t, this.globalOptions);
            if (t.did && !t.force)
                return;
            if (!ctx.globalOptions.loading) {
                console.log(chalk_1.default.yellow('Task: ') + t.name);
                let ret = t.fn && (yield t.fn(ctx));
                t.did = true;
                return ret;
            }
            ld = ora({
                text: chalk_1.default.gray(text),
            }).start();
            try {
                let ret = yield (t.fn && t.fn(ctx));
                ld.succeed(text);
                t.did = true;
                return ret;
            }
            catch (error) {
                ld.fail(chalk_1.default.redBright(text));
                throw error;
            }
        });
    }
}
exports.TaskManager = TaskManager;
const taskManager = new TaskManager();
const TMKey = '@foyjs/taskManager';
function getGlobalTaskManager() {
    return global[TMKey];
}
exports.getGlobalTaskManager = getGlobalTaskManager;
global[TMKey] = taskManager;
/**
 * Set global options for all tasks.
 * @param options
 */
function setGlobalOptions(options) {
    Object.assign(getGlobalTaskManager().globalOptions, options);
}
exports.setGlobalOptions = setGlobalOptions;
var TaskOptions;
(function (TaskOptions) {
    TaskOptions.last = empty();
    function empty() {
        return {
            desc: undefined,
            optionDefs: [],
            strict: getGlobalTaskManager().globalOptions.strict,
        };
    }
    TaskOptions.empty = empty;
})(TaskOptions || (TaskOptions = {}));
function desc(desc) {
    TaskOptions.last.desc = desc;
}
exports.desc = desc;
function option(rawName, description, config) {
    TaskOptions.last.optionDefs.push([rawName, description, config]);
}
exports.option = option;
function strict() {
    TaskOptions.last.strict = true;
}
exports.strict = strict;
/**
 * Set options for next task.
 * @param options
 */
function setOption(options) {
    Object.assign(TaskOptions.last, options);
}
exports.setOption = setOption;
/**
 * Define a task
 * @param name
 * @param dependencies
 * @param fn
 */
function task(name, dependencies, fn) {
    if (typeof dependencies === 'function') {
        fn = dependencies;
        dependencies = [];
    }
    const t = {
        name,
        options: {},
        optionDefs: TaskOptions.last.optionDefs,
        desc: TaskOptions.last.desc,
        strict: TaskOptions.last.strict,
        dependencies: (dependencies || []).map(d => {
            if (typeof d === 'string') {
                return { name: d, options: {} };
            }
            d.options = d.options || {};
            return d;
        }),
        fn,
    };
    TaskOptions.last = TaskOptions.empty();
    taskManager.addTask(t);
    return t;
}
exports.task = task;
//# sourceMappingURL=task.js.map